# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['builder']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.1,<8.0.0', 'grafanalib>=0.5.5,<0.6.0']

entry_points = \
{'console_scripts': ['grafana-dashboards-builder = builder.builder:build']}

setup_kwargs = {
    'name': 'grafana-dashboards-builder',
    'version': '0.2.0',
    'description': 'A wrapper around grafanalib which simplifies generating multiple dashboards.',
    'long_description': "# Grafana Dashboards Builder\n\nA wrapper around [grafanalib](https://github.com/weaveworks/grafanalib) which simplifies generating multiple dashboards.\n\n## Why?\n\nGrafanalib is a fantastic tool which lets you generate Grafana dashboards from simple Python scripts. Unfortunately, it can only read single files as dashboard sources and doesn't have a concept of multiple output directories. Those limitations make it hard to provision an entire Grafana instance with many folders and multiple dashboard sources.\n\nGrafana Dashboard Builder recursively finds all `.dashboard.py` files in a directory tree. It generates dashboards and places them in a subdirectories which represent [Grafana folders](https://grafana.com/docs/grafana/latest/reference/dashboard_folders/).\n\nIt's written with Kubernetes in mind so it can also generate nested output directory structure even when loading sources mounted from a flat ConfigMap (see [examples](#sources-in-a-configmap) below).\n\n\n## Installation\n\n    pip install grafana-dashboards-builder\n\n\n## Usage\n\n    grafana-dashboards-builder [OPTIONS] INPUT_DIR [OUTPUT_DIR]\n\n    INPUT_DIR is the directory tree with dashboard sources.\n\n    OUTPUT_DIR is the directory where generated dashboards are placed (defaults to ./out).\n\n    Options:\n    --from-configmap  generate output directories based on a source files prefix and a '--' separator\n    --help            Show this message and exit.\n\n\n## Examples\n\n### Sources in nested directories\n\nGrafana supports only one level of depth for [folders](https://grafana.com/docs/grafana/latest/reference/dashboard_folders/). So even when dashboard sources are nested in multiple subdirectories the output dashboards will have only one parent directory (the most shallow one). Source dashboards that don't have any parent directory will be placed in the default `General` folder.\n\nFor example, given a following input directory tree:\n\n    dashboards_in/\n        main.dashboard.py\n        kubernetes/\n            workloads/\n                pods.dashboard.py\n                jobs.dashboard.py\n            nodes/\n                nodes.dashboard.py\n        nginx/\n            nginx_health.dashboard.py\n\nA following output directory will be generated:\n\n    dashboards_out/\n        General/\n            main.json\n        Kubernetes/\n            pods.json\n            jobs.json\n            nodes.json\n        Nginx/\n            nginx_health.json\n\n### Sources in a ConfigMap\n\nWhen running Grafana in Kubernetes cluster it's possible to run Grafana Dashboard Builder as a sidecar which loads dashboard sources from a ConfigMap and generates them into Grafana's `/var/lib/grafana/dashboards` directory.\n\nConfigMaps don't support nested directory structures so to enable mapping dashboards to different folders we can prefix the sources filenames with a `folder--` prefix. When Grafana Dashboard Builder runs with `--from-configmap` flag, it parses the filenames and generates output directories based on found prefixes. Filenames without a prefix will be placed in the default `General` folder.\n\nFor example, given a following input directory:\n\n    dashboards_in/\n        main.dashboard.py\n        kubernetes--pods.dashboard.py\n        kubernetes--jobs.dashboard.py\n        nginx--nginx_health.dashboard.py\n        \nA following output directory will be generated:\n\n    dashboards_out/\n        General/\n            main.json\n        Kubernetes/\n            pods.json\n            jobs.json\n        Nginx/\n            nginx_health.json\n",
    'author': 'Grzegorz Dlugoszewski',
    'author_email': 'pypi@grdl.dev',
    'maintainer': 'Grzegorz Dlugoszewski',
    'maintainer_email': 'pypi@grdl.dev',
    'url': 'https://gitlab.com/grdl/grafana-dashboards-builder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import click
import shutil
from pathlib import Path
from grafanalib import _gen as generator

# Default folder in Grafana where dashboards without a parent directory go
DEFAULT_FOLDER = 'General'

# Default output directory for generated dashboard json files
DEFAULT_OUT = 'out'

# Separator used to split directory name from filename when using "--from-configmap" flag (ie. without nested input directories)
DIR_SEPARATOR = '--'


@click.command()
@click.argument('input-dir', type=click.Path(file_okay=False, exists=True))
@click.argument('output-dir', type=click.Path(file_okay=False), default=DEFAULT_OUT)
@click.option('--from-configmap', is_flag=True, default=False, help="generate output directories based on a source files prefix and a '--' separator")
def build(input_dir, output_dir, from_configmap):
    """
    Grafana Dashboards Builder - a wrapper around grafanalib which simplifies generating multiple dashboards.

    INPUT_DIR is the directory tree with dashboard sources.

    OUTPUT_DIR is the directory where generated dashboards are placed (defaults to ./out).
    """

    # TODO: can this be checked in click.argument callback?
    if input_dir == output_dir:
        raise click.BadArgumentUsage("'OUTPUT_DIR' can't be the same as 'INPUT_DIR'.")

    dashboards = load_dashboards(input_dir, from_configmap)

    if len(dashboards) == 0:
        click.echo("There are no dashboards to generate")
        return

    generate_dashboards(dashboards, output_dir)


def load_dashboards(input_dir, from_configmap=False):
    """
    Load .dashboard.py files into a dictionary where keys are first level directories.
    Keys represent Grafana folders, but since Grafana allows only a single level of folder nesting we "flatten" dashboard.py files
    even if they're nested in multiple directories.

    :param input_dir str: Path to directory containing dashboard sources.

    :Example:

    For a following folder structure...

        input_dir/
            dash0.dashboard.py
            dir1/
                dash1.dashboard.py
            dir2/
                dash2.dashboard.py
                dir3/
                    dash3.dashboard.py

    ...a loaded dashboards dictionary should look like this:

        dashboards = {
            'DEFAULT_FOLDER': [dash0],
            'dir1': [dash1],
            'dir2': [dash2, dash3]
        }
    """

    dashboards = {}

    for path in Path(input_dir).rglob(f'*{generator.DASHBOARD_SUFFIX}'):
        try:
            dashboard = generator.load_dashboard(str(path))
        except generator.DashboardError:  # raised when file does not define 'dashboard' attribute
            continue
        except SyntaxError:  # raised when file contains invalid Python code
            continue

        # Key is the first level directory inside the input_dir in the dashboard path
        # If there are more nested dirs inside it, the key will still be the first-level one
        key = path.relative_to(input_dir).parts[0]

        # If key is the same as filename it means file is not nested in any directory but directly under the input_dir
        # If the --from-configmap flag is set, we extract the directory from filename (eg: "dirname--dash1.dashboard.py")
        # Otherwise it goes to the default "General" Grafana folder
        if key == path.name:
            key = DEFAULT_FOLDER

            if from_configmap and DIR_SEPARATOR in path.name:
                key = path.name.split(DIR_SEPARATOR, 1)[0]

        if key not in dashboards:
            dashboards[key] = []

        dashboards[key].append(dashboard)

    return dashboards


def generate_dashboards(dashboards, output_dir):
    """
    Generate and write dashboard json files into the output_dir. For each key in dashboards dict a new subfolder is created.
    If output_dir already exists it will be removed and recreated.
    """

    shutil.rmtree(output_dir, ignore_errors=True)   # Remove the output dir, don't raise errors if dir doesn't exist
    output_path = Path(output_dir)
    output_path.mkdir()

    for key in dashboards:
        subdir = output_path / key
        subdir.mkdir(exist_ok=True)

        for dashboard in dashboards[key]:
            output_file = output_path / key / f'{dashboard.title}.json'
            with open(output_file, 'w') as output:
                generator.write_dashboard(dashboard, output)


if __name__ == '__main__':
    build()  # pylint: disable=no-value-for-parameter, unexpected-keyword-arg
